"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
const MotionType_1 = __importDefault(require("../enums/MotionType"));
const quaternionA2T_1 = __importDefault(require("../utils/quaternionA2T"));
const quaternionT2A_1 = __importDefault(require("../utils/quaternionT2A"));
const vector3A2T_1 = __importDefault(require("../utils/vector3A2T"));
const vector3T2A_1 = __importDefault(require("../utils/vector3T2A"));
const XRClass_1 = __importDefault(require("./XRClass"));
class XRMass extends (0, XRClass_1.default)(three_1.Mesh) {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { hasBody: false, mass: 0, breakable: false, fragmentDepth: 0, fractureImpulse: 1 });
    }
    init() {
        this.initGeometry();
        this.materialStack = [];
        if (!this.material)
            this.material = new three_1.MeshBasicMaterial({ color: 0xffffff });
        this.materialStack.push(this.material);
    }
    initGeometry() {
    }
    update(p) {
        super.update(p);
        if (this.body) {
            if (this.body.isKinematicObject() && this.needsUpdate) {
                this._applyThreeToAmmo();
                this.needsUpdate = false;
            }
            else if (!this.body.isStaticObject()) {
                this._applyAmmoToThree();
            }
        }
    }
    setAngularVelocity(x = 0, y = 0, z = 0) {
        let v = x instanceof three_1.Vector3 ? (0, vector3T2A_1.default)(x) : new Ammo.btVector3(x, y, z);
        this.body.setAngularVelocity(v);
        Ammo.destroy(v);
    }
    setLinearVelocity(x = 0, y = 0, z = 0) {
        let v = x instanceof three_1.Vector3 ? (0, vector3T2A_1.default)(x) : new Ammo.btVector3(x, y, z);
        this.body.setLinearVelocity(v);
        Ammo.destroy(v);
    }
    setMaterial(m) {
        var _a, _b, _c, _d, _e, _f;
        this.materialStack.push(m);
        if (Array.isArray(this.material)) {
            if (((_a = m.userData) === null || _a === void 0 ? void 0 : _a.importance) >= ((_c = (_b = this.material[0]) === null || _b === void 0 ? void 0 : _b.userData) === null || _c === void 0 ? void 0 : _c.importance) || 0)
                this.material[0] = m;
        }
        else {
            if (((_d = m.userData) === null || _d === void 0 ? void 0 : _d.importance) >= (((_f = (_e = this.material) === null || _e === void 0 ? void 0 : _e.userData) === null || _f === void 0 ? void 0 : _f.importance) || 0))
                this.material = m;
        }
    }
    removeMaterial(m, importance = 0) {
        this.materialStack = this.materialStack.filter(mm => mm !== m);
        if (this.material === m) {
            if (this.materialStack.length)
                this.material = this.materialStack[this.materialStack.length - 1];
        }
    }
    turnOnTempKinematics() {
        const b = this.body;
        this.userData.prevCollisionFlags = b === null || b === void 0 ? void 0 : b.getCollisionFlags();
        if (b && b.getCollisionFlags() !== MotionType_1.default.Kinematic)
            b.setCollisionFlags(MotionType_1.default.Kinematic);
    }
    turnOffTempKinematics() {
        if (this.userData.prevCollisionFlags === MotionType_1.default.Kinematic)
            return;
        const b = this.body;
        if (!b)
            return;
        this._applyThreeToAmmo();
        b.setCollisionFlags(this.userData.prevCollisionFlags);
        if (this.userData.jiggle) {
            this.setLinearVelocity(0, 1, 0);
        }
        else {
            this.stopVelocities();
        }
        this.userData.jiggle = false;
    }
    stopVelocities() {
        const b = this.body;
        const v = new Ammo.btVector3(0, 0, 0);
        b === null || b === void 0 ? void 0 : b.setLinearVelocity(v);
        b === null || b === void 0 ? void 0 : b.setAngularVelocity(v);
        Ammo.destroy(v);
    }
    _applyThreeToAmmo() {
        const b = this.body;
        const ammoState = b.getMotionState();
        const ammoT = new Ammo.btTransform();
        ammoState.getWorldTransform(ammoT);
        const thisQ = this.getWorldQuaternion(new three_1.Quaternion());
        const thisP = this.getWorldPosition(new three_1.Vector3());
        const thisS = this.getWorldScale(new three_1.Vector3());
        b.getCollisionShape().setLocalScaling((0, vector3T2A_1.default)(thisS));
        ammoT.setOrigin((0, vector3T2A_1.default)(thisP));
        ammoT.setRotation((0, quaternionT2A_1.default)(thisQ));
        ammoState.setWorldTransform(ammoT);
        Ammo.destroy(ammoT);
    }
    _applyAmmoToThree() {
        const ammoState = this.body.getMotionState();
        const ammoT = new Ammo.btTransform();
        ammoState.getWorldTransform(ammoT);
        const physP = ammoT.getOrigin();
        const physQ = ammoT.getRotation();
        const scale = this.body.getCollisionShape().getLocalScaling();
        const thisS = (0, vector3A2T_1.default)(scale);
        const thisP = (0, vector3A2T_1.default)(physP);
        const thisQ = (0, quaternionA2T_1.default)(physQ);
        const thisM = new three_1.Matrix4().compose(thisP, thisQ, thisS);
        new three_1.Matrix4()
            .copy(this.parent.matrixWorld)
            .invert()
            .multiply(thisM)
            .decompose(this.position, this.quaternion, this.scale);
        Ammo.destroy(ammoT);
    }
}
exports.default = XRMass;
//# sourceMappingURL=XRMass.js.map