"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const debounce_1 = __importDefault(require("lodash/debounce"));
const three_1 = require("three");
const MotionType_1 = __importDefault(require("../enums/MotionType"));
const ParamType_1 = __importDefault(require("../enums/ParamType"));
const applyPosQuatScaleMargin_1 = __importDefault(require("../utils/applyPosQuatScaleMargin"));
const collisionShapeToRigidBody_1 = __importDefault(require("../utils/collisionShapeToRigidBody"));
const createCollisionShape_1 = __importDefault(require("../utils/createCollisionShape"));
const createPhysicsWorld_1 = __importDefault(require("../utils/physics/createPhysicsWorld"));
const destroyPhysicsWorld_1 = __importDefault(require("../utils/physics/destroyPhysicsWorld"));
const vector3T2A_1 = __importDefault(require("../utils/vector3T2A"));
const XRClass_1 = __importDefault(require("./XRClass"));
const XRMass_1 = __importDefault(require("./XRMass"));
const XRObject_1 = __importDefault(require("./XRObject"));
const XRPlayer_1 = __importDefault(require("./XRPlayer"));
class XRScene extends (0, XRClass_1.default)() {
    constructor() {
        super(...arguments);
        this.reinit = (0, debounce_1.default)(() => {
            this.build();
        }, 700);
    }
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { label: 'XRScene', playerInitPosition: new three_1.Vector3(0, 0, 10), playerInitRotation: new three_1.Vector3(0, 0, 0), gravity: new three_1.Vector3(0, -9.81, 0), params: {}, controllerBehaviours: [], objects: [], masses: [], lights: [] });
    }
    init() {
        super.init();
        this.physxScene = (0, createPhysicsWorld_1.default)((0, vector3T2A_1.default)(this.gravity));
        Object.keys(this.params).forEach(k => (this.params[k].key = k));
        this.gfxScene = new three_1.Scene();
        this.build();
    }
    build() { }
    update(p) {
        super.update(p);
        if (p === null || p === void 0 ? void 0 : p.delta)
            this.physxScene.stepSimulation(p === null || p === void 0 ? void 0 : p.delta);
        void Promise.all([
            ...this.objects.map(o => o === null || o === void 0 ? void 0 : o.update(p)),
            ...this.masses.map(o => o === null || o === void 0 ? void 0 : o.update(p)),
            ...this.lights.map(o => o === null || o === void 0 ? void 0 : o.update(p)),
        ]);
    }
    setParam(param, value) {
        if (param.type === ParamType_1.default.BUTTON)
            return void param.callback(this);
        this.params[param.key].value = value;
        this.destroy(false);
        this.reinit();
    }
    get all() {
        return [...this.objects, ...this.masses, ...this.lights];
    }
    addLight(l) {
        this.lights.push(l);
        this.gfxScene.add(l);
    }
    addObject(o) {
        this.objects.push(o);
        this.gfxScene.add(o);
    }
    addMass(o, type = MotionType_1.default.Dynamic) {
        this.masses.push(o);
        this.gfxScene.add(o);
        const pos = o.getWorldPosition(new three_1.Vector3());
        const quat = o.getWorldQuaternion(new three_1.Quaternion());
        const scale = o.getWorldScale(new three_1.Vector3());
        const collisionGroup = 1;
        const collisionMask = -1;
        const collisionShape = (0, createCollisionShape_1.default)(o);
        const localTransform = (0, applyPosQuatScaleMargin_1.default)(collisionShape, pos, quat, scale);
        const rigidBody = (0, collisionShapeToRigidBody_1.default)(collisionShape, localTransform, o.mass, type === MotionType_1.default.Kinematic);
        o.body = rigidBody;
        this.physxScene.addRigidBody(rigidBody, collisionGroup, collisionMask);
    }
    remove(o) {
        this.gfxScene.remove(o);
        if (o instanceof XRMass_1.default) {
            if (o === null || o === void 0 ? void 0 : o.body) {
                this.physxScene.removeRigidBody(o.body);
                Ammo.destroy(o.body);
                delete o.body;
            }
            this.masses = this.masses.filter(m => m !== o);
        }
        else if (o instanceof XRObject_1.default) {
            this.objects = this.objects.filter(m => m !== o);
        }
        else {
            this.lights = this.lights.filter(l => l !== o);
        }
    }
    destroy(fullyDestroy = true) {
        const player = this.objects.find(v => v instanceof XRPlayer_1.default);
        this.objects.filter(v => fullyDestroy || !(v === player)).forEach(this.remove.bind(this));
        this.masses.forEach(this.remove.bind(this));
        this.lights.forEach(this.remove.bind(this));
        if (fullyDestroy)
            (0, destroyPhysicsWorld_1.default)(this.physxScene);
        this.objects = [];
        this.masses = [];
        this.lights = [];
        if (!fullyDestroy)
            this.objects.push(player);
        if (fullyDestroy)
            this.gfxScene = undefined;
        if (fullyDestroy)
            this.controllerBehaviours = [];
        if (fullyDestroy)
            this.params = {};
    }
}
exports.default = XRScene;
//# sourceMappingURL=XRScene.js.map